import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

import konstanten.Konst;
import animation.EinachserAnimation;

import simulator.Simulator;
import integrator.*;
import modell.*;

import evoopt.*;

public class Einachser extends PApplet
{
    Graycode graycode = new Graycode(this);
    Zufall zufall = new Zufall(this,0); //immer gleicher seed!!!

    PApplet pap = this;
    
    int SPEED = 200;

    double phi1=0.0;
    double phi2=0.0;

    int position = 0;
    boolean AKTIV=false;
    Button knopf;
    
    EinachserAnimation animation;
    
    EinachserModell modell;
    RuKuIntegrator    integrator;
    Simulator simulator;
    
    double[] parameter = {500.0,10.0};
    double[] min_parameter = {0.0,0.0};
    double[] max_parameter = {1000.0,100.0};
//    double[] anfangsbedingungen = {0.0, 0.0, Math.PI-0.5, 0.0};
//    double[] anfangsbedingungen = {0.0, 0.0, Math.PI*0.5, 0.0};
    double[] anfangsbedingungen = {0.0, 0.0, 0.5, 0.0, 0.0};
    
    int index=0;
    
    public void setup()
    {
        setLayout(null);
        knopf = new Button("Animation");
        knopf.setBounds(10,300,120,30);
        knopf.addActionListener(new Knopflauscher());
        add(knopf);        
        
        modell = new EinachserModell();
        modell.setzeParameter(parameter);
        integrator = new RuKuIntegrator();
        integrator.registriereModell(modell);
        simulator = new Simulator(modell,integrator);
        
        animation = new EinachserAnimation(this);
        
        size(Konst.BREITE,Konst.HOEHE);
        frameRate(10);
        
    }

    public void draw()
    {
        clear();
        strokeWeight(2.0f);
        
        animation.draw(phi1,phi2);
        
        if(AKTIV)
        {            
            phi1=simulator.y[0][index];
            phi2=simulator.y[2][index];
                            
            index+=SPEED;
            if(index>=simulator.t.length)
                index=0;
        }

        knopf.repaint();
        
    }
    

    
    private class Knopflauscher implements ActionListener
    {
        public void actionPerformed(ActionEvent ee)
        {
            if(!AKTIV)
            {
                index=0;
                FehlerEinachser fehlerfunktion = new FehlerEinachser();
                int anzahlgene = 30;
                int anzahlbeste = 5;
                int anzahlstellen = 48; //Anzahl der Bits der Gene
                int sooftmutieren = 20; 
                System.out.println("Generation erzeugen");
                Generation generation = new Generation(pap,anzahlgene,anzahlbeste,anzahlstellen,sooftmutieren,zufall,fehlerfunktion);                

                //Optimierung durchf&uuml;hren:
                System.out.println("Optimierung beginnen");
                int MAXDURCHLAEUFE = 30;
                for(int i=0;i<MAXDURCHLAEUFE;i++)
                {
                    System.out.println("BESTES ERGEBNIS BEI DURCHLAUF Nr."+i+":");
                    Gen bestesgen = generation.beste[0];
                    fehlerfunktion.berechneUndZeige(bestesgen);
                    if(i<MAXDURCHLAEUFE-1)
                        generation.neueGeneration();
                }                        
                parameter[0] = fehlerfunktion.testparameter[0];
                parameter[1] = fehlerfunktion.testparameter[1];
                simulator.simulieren(parameter,anfangsbedingungen,0.0,2.0,0.00002);                
                
                knopf.setBackground(Color.RED);
            }   
            else
            {
                knopf.setBackground(Color.GREEN);
            }
            AKTIV=!AKTIV;
        }        
    }
    
    public static void main(String[] passedArgs) 
    {
        String[] appletArgs = new String[] { "Einachser" };
        if(passedArgs != null) 
        {
             PApplet.main(concat(appletArgs, passedArgs));
        } 
        else 
        {
            PApplet.main(appletArgs);
        }            
    }
    
    public class FehlerEinachser implements Fehlerfunktion
    {        
        public double[] testparameter = {0.0,0.0};    

        public double berechne(Gen gen)
        {
            //gray2doublearr(double[] dwert, int[] gray, double[] dminwert, double[] dmaxwert);
            graycode.gray2doublearr(testparameter,gen.arr,min_parameter, max_parameter);
            simulator.simulieren(testparameter,anfangsbedingungen,0.0,0.1,0.00002);            
            double FEHLER = 0.0;
            for(int i=0;i<simulator.y[2].length;i++)
                FEHLER+=simulator.y[2][i]*simulator.y[2][i];
            return FEHLER;
        }
        public double berechneUndZeige(Gen gen)
        {
            //gray2doublearr(double[] dwert, int[] gray, double[] dminwert, double[] dmaxwert);
            graycode.gray2doublearr(testparameter,gen.arr,min_parameter, max_parameter);
            simulator.simulieren(testparameter,anfangsbedingungen,0.0,0.1,0.00002);            
            double FEHLER = 0.0;
            for(int i=0;i<simulator.y[2].length;i++)
                FEHLER+=simulator.y[2][i]*simulator.y[2][i];
                
            System.out.println("AKTUELLER FEHLER:"+FEHLER);    
            System.out.println("P_ANTEIL:"+testparameter[0]);    
            System.out.println("D_ANTEIL:"+testparameter[1]);    
            return FEHLER;
        }
    }
}
