package animation;

import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 


import hilfsklassen.Linien;
import konstanten.Konst;

/**
 * KÖRPER1 (Rad)
 * m1=0,094kg
 * r = 0,035m Radradius
 * J1=0,0000576 kgm2
 * 
 * KÖRPER2 (Rumpf)
 * m2 = 0,364kg
 * d = 0,06m     Abstand Achse zu Rumpfschwerpunkt
 * b = 0,057m    Rumpf-Breite
 * h = 0,14m     Rumpf-Höhe
 * J2=0,000693 kgm2
 *
 * Konvention: 1Pixel == 5mm
 *             200Pixel ==1m
 *
 */
public class EinachserAnimation
{
    private PApplet pap;
        
    private Linien rad;
    private Linien rumpf;
    private Linien boden;
    
    public EinachserAnimation(PApplet pap)
    {
        this.pap = pap;
        
        rad = new Linien(pap);        
        rumpf = new Linien(pap);
        boden = new Linien(pap);
        
        //Körper mit Ursprung im Schwerpunkt definieren:
        //   _1
        // 4| |2
        //   -
        //   3
        rumpf.add( Konst.b*0.5,Konst.h-Konst.d,   //1
                  -Konst.b*0.5,Konst.h-Konst.d);
        rumpf.add( Konst.b*0.5,Konst.h-Konst.d,   //2
                   Konst.b*0.5,-Konst.d);     
        rumpf.add( Konst.b*0.5,-Konst.d,          //3
                  -Konst.b*0.5,-Konst.d);
        rumpf.add( -Konst.b*0.5,Konst.h-Konst.d, //4
                   -Konst.b*0.5,-Konst.d);
                   
        //Rad mit Ursprung im Schwerpunkt definieren:
        //a) Speichen
        double beta = 0.0;
        for(int i=0;i<8;i++)
        {
            double xx = Konst.r*Math.cos(beta);
            double yy = Konst.r*Math.sin(beta);
            
            rad.add(0.0,0.0,xx,yy);
            
            beta+=2.0*Math.PI/8.0;
        }
        //b) Umfang
        beta = 0.0;
        double xx_alt = Konst.r;
        double yy_alt = 0.0;
        for(int i=1;i<=25;i++)
        {
            double xx = Konst.r*Math.cos(beta);
            double yy = Konst.r*Math.sin(beta);
            
            rad.add(xx_alt,yy_alt,xx,yy);
            
            beta+=2.0*Math.PI/24.0;
            xx_alt = xx;
            yy_alt = yy;
        }
        
        //Boden
        boden.add(-2.0,0.0,2.0,0.0);
    }

    public void draw(double phi1,double phi2)
    {
        //Koordinatentransformationen durchführen:
        double xs1 = -phi1*Konst.r;
        double ys1 = Konst.r;
        
        double xs2 = xs1 - Math.sin(phi2)*Konst.d;
        double ys2 = ys1 + Math.cos(phi2)*Konst.d;
        
        boden.transformToInert(0.0,0.0,0.0);
        boden.transformToPixel(Konst.skalierung,Konst.XOFF,Konst.YOFF,Konst.BREITE,Konst.HOEHE);

        rad.transformToInert(xs1,ys1,phi1);
        rad.transformToPixel(Konst.skalierung,Konst.XOFF,Konst.YOFF,Konst.BREITE,Konst.HOEHE);
        
        rumpf.transformToInert(xs2,ys2,phi2);
        rumpf.transformToPixel(Konst.skalierung,Konst.XOFF,Konst.YOFF,Konst.BREITE,Konst.HOEHE);
        
        boden.draw(0,0,255);
        rumpf.draw(0,255,0);
        rad.draw(255,0,0);
    }
}
